package com.qa.utilities;

import java.io.File;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.stream.Stream;

import org.apache.commons.io.FileUtils;
import org.codehaus.jackson.JsonNode;
import org.codehaus.jackson.JsonParseException;
import org.codehaus.jackson.JsonProcessingException;
import org.codehaus.jackson.map.JsonMappingException;
import org.codehaus.jackson.map.ObjectMapper;
import org.codehaus.jackson.node.ObjectNode;

import com.google.common.base.Charsets;

import io.restassured.RestAssured;
import io.restassured.http.ContentType;
import io.restassured.path.json.JsonPath;
import io.restassured.response.Response;
import io.restassured.specification.RequestSpecification;
import static io.restassured.RestAssured.given;

public class RestUtil {
	// Global Setup Variables
	public static String path; // Rest request path
	static RequestSpecification request = RestAssured.given();

	/*
	 *** Sets Base URI*** Before starting the test, we should set the
	 * RestAssured.baseURI
	 */
	public static void setBaseURI(String baseURI) {
		RestAssured.baseURI = baseURI;
	}

	/*
	 *** Sets ContentType*** We should set content type as JSON or XML before starting
	 * the test
	 */
	public static void setContentType(String Type) {
		request.with().contentType(Type);
	}

	/*
	 *** search query path of first example*** It is equal to
	 * "barack obama/videos.json?num_of_videos=4"
	 */
	public static String createSearchQueryPath(String searchTerm, String jsonPathTerm, String param,
			String paramValue) {
		path = "?" + param + "=" + paramValue;
		return path;
	}

	/*
	 *** Returns response*** We send "path" as a parameter to the Rest Assured'a "get"
	 * method and "get" method returns response of API
	 */
	public static Response getResponse() {
		// System.out.print("path: " + path +"\n");
		return RestAssured.get(path);
	}

	/*
	 *** Returns JsonPath object*** First convert the API's response to String type
	 * with "asString()" method. Then, send this String formatted json response to
	 * the JsonPath class and return the JsonPath
	 */
	public static JsonPath getJsonPath(Response res) {
		String json = res.asString();
		// System.out.print("returned json: " + json +"\n");
		return new JsonPath(json);
	}

	public static void setreqPayload(File file) {
		request.given().body(file);
	}

	public static void setreqPayloadAddRandomName(File file) throws JsonProcessingException, IOException {
       //String with random generated name
		String randomName = "Test Post call "+ Math.random();
		
		//read the Json Payload from file
		String JSONObjectStr = readLineByLineJava8(file.getPath());
		//String JSONObjectStr = FileUtils.readFileToString(file.getPath());
		ObjectMapper mapper = new ObjectMapper();
		
		//JsonNode will have deserialized Json 
		JsonNode actualObj = mapper.readTree(JSONObjectStr);
		ObjectNode jsonObjectNode = (ObjectNode) actualObj;
		
		//put the random value in jsonObjectNode
		jsonObjectNode.put("name", randomName);
		
		// write jsonObjectNode as strings
		
		String jsonInString = mapper.writeValueAsString(jsonObjectNode);
		request.given().body(jsonInString);
	}
	
	private static String readLineByLineJava8(String filePath)
	{
	    StringBuilder contentBuilder = new StringBuilder();
	    try (Stream<String> stream = Files.lines( Paths.get(filePath), StandardCharsets.UTF_8))
	    {
	        stream.forEach(s -> contentBuilder.append(s).append("\n"));
	    }
	    catch (IOException e)
	    {
	        e.printStackTrace();
	    }
	    return contentBuilder.toString();
	}
	
	
	public static Response setPostwebserviceUrl(String baseuri, String resourceuri) {
		String url = baseuri.concat(resourceuri);
		url = url.replaceAll("\"", "");
		System.out.println(url);
		return request.when().post(url);
	}

	public static void setGetResourceUri(String resourceuri) {
		RestAssured.when().get(resourceuri);
	}

	public static Response setGETEndPoint(String baseuri, String resourceuri, String driverId) {
		return request.when().baseUri(baseuri).get(resourceuri + "/" + driverId + ".json");
	}

	public static void setpathParams(String driverId) {
		request.pathParams("driverId", driverId + ".json");
	}
}
